@@ -70,7 +70,7 @@ int (*parse_ipaddr) (const char *restrict src, union ipaddr_t *restrict dst);
 void (*copy_ipaddr) (union ipaddr_t *restrict dst, const union ipaddr_t *restrict src);
 const char *(*ntop_ipaddr) (const union ipaddr_t *ip);
 double (*get_range_size) (const struct range_t *r);
-int (*xstrstr) (const char *__restrict str);
+int (*xstrstr) (const char *restrict str);
 int (*ipcomp) (const union ipaddr_t *restrict a, const union ipaddr_t *restrict b);
 int (*leasecomp) (const struct leases_t *restrict a, const struct leases_t *restrict b);
 int (*output_analysis) (void);
@@ -98,10 +98,9 @@ static int return_limit(const char c)
  * alarming. */
 int main(int argc, char **argv)
 {
-	int i;
 	int option_index = 0;
 	char const *tmp;
-	char *print_mac_addreses_tmp;
+	const char *print_mac_addreses_tmp;
 	struct range_t *tmp_ranges;
 	enum {
 		OPT_SNET_ALARMS = CHAR_MAX + 1,
@@ -109,7 +108,8 @@ int main(int argc, char **argv)
 		OPT_CRIT,
 		OPT_MINSIZE,
 		OPT_WARN_COUNT,
-		OPT_CRIT_COUNT
+		OPT_CRIT_COUNT,
+		OPT_SET_IPV
 	};
 	int ret_val;
 
@@ -129,6 +129,9 @@ int main(int argc, char **argv)
 		{"warn-count", required_argument, NULL, OPT_WARN_COUNT},
 		{"crit-count", required_argument, NULL, OPT_CRIT_COUNT},
 		{"minsize", required_argument, NULL, OPT_MINSIZE},
+		{"perfdata", no_argument, NULL, 'p'},
+		{"all-as-shared", no_argument, NULL, 'A'},
+		{"ip-version", required_argument, NULL, OPT_SET_IPV},
 		{NULL, 0, NULL, 0}
 	};
 
@@ -143,11 +146,12 @@ int main(int argc, char **argv)
 	 * command line option */
 	config.output_file[0] = '\0';
 	/* Alarming defaults. */
-	config.snet_alarms = false;
+	config.snet_alarms = 0;
 	config.warning = ALARM_WARN;
 	config.critical = ALARM_CRIT;
 	config.warn_count = 0x100000000;	/* == 2^32 that is the entire IPv4 space */
 	config.crit_count = 0x100000000;	/* basically turns off the count criteria */
+	config.perfdata = 0;
 	/* File location defaults */
 	strncpy(config.dhcpdconf_file, DHCPDCONF_FILE, MAXLEN - 1);
 	strncpy(config.dhcpdlease_file, DHCPDLEASE_FILE, MAXLEN - 1);
@@ -158,14 +162,17 @@ int main(int argc, char **argv)
 	/* Make sure some output format is selected by default */
 	print_mac_addreses_tmp = OUTPUT_FORMAT;
 	/* Default sort order is by IPs small to big */
-	config.reverse_order = false;
-	config.backups_found = false;
+	config.reverse_order = 0;
+	config.backups_found = 0;
+	/* Treat single networks as shared with network CIDR as name */
+	config.all_as_shared = 0;
 	prepare_memory();
+	set_ipv_functions(IPvUNKNOWN);
 	/* Parse command line options */
 	while (1) {
 		int c;
 
-		c = getopt_long(argc, argv, "c:l:f:o:s:rL:vh", long_options, &option_index);
+		c = getopt_long(argc, argv, "c:l:f:o:s:rL:pAvh", long_options, &option_index);
 		if (c == EOF)
 			break;
 		switch (c) {
@@ -185,10 +192,11 @@ int main(int argc, char **argv)
 		{
 			/* Output sorting option */
 			struct output_sort *p = config.sorts;
+			size_t len;
 
 			while (p && p->next)
 				p = p->next;
-			for (i = 0; i < strlen(optarg); i++) {
+			for (len = 0; len < strlen(optarg); len++) {
 				if (config.sorts == NULL) {
 					config.sorts = xcalloc(1, sizeof(struct output_sort));
 					p = config.sorts;
@@ -196,13 +204,13 @@ int main(int argc, char **argv)
 					p->next = xcalloc(1, sizeof(struct output_sort));
 					p = p->next;
 				}
-				p->func = field_selector(optarg[i]);
+				p->func = field_selector(optarg[len]);
 			}
 		}
 			break;
 		case 'r':
 			/* What ever sort in reverse order */
-			config.reverse_order = true;
+			config.reverse_order = 1;
 			break;
 		case 'o':
 			/* Output file */
@@ -214,7 +222,7 @@ int main(int argc, char **argv)
 			config.number_limit = return_limit(optarg[1]);
 			break;
 		case OPT_SNET_ALARMS:
-			config.snet_alarms = true;
+			config.snet_alarms = 1;
 			break;
 		case OPT_WARN:
 			print_mac_addreses_tmp = "a";
@@ -235,6 +243,26 @@ int main(int argc, char **argv)
 		case OPT_MINSIZE:
 			config.minsize = strtod_or_err(optarg, "illegal argument");
 			break;
+		case OPT_SET_IPV:
+			switch(optarg[0]) {
+			case '4':
+				set_ipv_functions(IPv4);
+				break;
+			case '6':
+				set_ipv_functions(IPv6);
+				break;
+			default:
+				error(EXIT_FAILURE, 0, "unknown --ip-version argument: %s", optarg);
+			}
+			break;
+		case 'p':
+			/* Print additional performance data in alarming mode */
+			config.perfdata = 1;
+			break;
+		case 'A':
+			/* Treat single networks as shared with network CIDR as name */
+			config.all_as_shared = 1;
+			break;
 		case 'v':
 			/* Print version */
 			print_version();
@@ -288,15 +316,14 @@ int main(int argc, char **argv)
 		error(EXIT_FAILURE, 0, "unknown output format: %s", quote(print_mac_addreses_tmp));
 	}
 	/* Do the job */
-	set_ipv_functions(IPvUNKNOWN);
-	parse_config(true, config.dhcpdconf_file, shared_networks);
+	parse_config(1, config.dhcpdconf_file, shared_networks);
 	parse_leases();
 	prepare_data();
 	do_counting();
 	tmp_ranges = xmalloc(sizeof(struct range_t) * num_ranges);
 	if (config.sorts != NULL)
 		mergesort_ranges(ranges, num_ranges, tmp_ranges);
-	if (config.reverse_order == true)
+	if (config.reverse_order == 1)
 		flip_ranges(ranges, tmp_ranges);
 	free(tmp_ranges);
 	ret_val = output_analysis();
