@@ -63,21 +63,6 @@
 #  define unlikely(x)	(x)
 # endif
 
-/* The __attribute__ feature is available in gcc versions 2.5 and later.
- * The attribute __pure__ was added in gcc 2.96.  */
-# if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 96)
-#  define _DP_ATTRIBUTE_PURE __attribute__ ((__pure__))
-# else
-#  define _DP_ATTRIBUTE_PURE	/* empty */
-# endif
-
-/* The __const__ attribute was added in gcc 2.95.  */
-# if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
-#  define _DP_ATTRIBUTE_CONST __attribute__ ((__const__))
-# else
-#  define _DP_ATTRIBUTE_CONST	/* empty */
-# endif
-
 /* The attribute __hot__ was added in gcc 4.3.  */
 # if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
 #  define _DP_ATTRIBUTE_HOT __attribute__ ((__hot__))
@@ -123,6 +108,7 @@ struct shared_network_t {
 	double used;
 	double touched;
 	double backups;
+	int netmask;
 };
 /*! \struct range_t
  * \brief Counters for an individual range.
@@ -143,7 +129,9 @@ enum isc_conf_parser {
 	ITS_A_RANGE_FIRST_IP,
 	ITS_A_RANGE_SECOND_IP,
 	ITS_A_SHAREDNET,
-	ITS_AN_INCLUCE
+	ITS_AN_INCLUCE,
+	ITS_A_SUBNET,
+	ITS_A_NETMASK
 };
 /*! \enum ltype
  * \brief Lease state types.
@@ -210,6 +198,8 @@ struct configuration_t {
 		backups_found:1,
 		snet_alarms:1,
 		print_mac_addreses:1,
+		perfdata:1,
+		all_as_shared:1,
 		header_limit:3,
 		number_limit:3;
 };
@@ -235,54 +225,56 @@ extern unsigned int RANGES;
 extern void prepare_memory(void);
 extern void set_ipv_functions(int version);
 extern int parse_leases(void);
-extern void parse_config(int, const char *__restrict, struct shared_network_t *__restrict)
+extern void parse_config(int, const char *restrict, struct shared_network_t *restrict)
     __attribute__ ((nonnull(2, 3)));
 extern void prepare_data(void);
 extern void do_counting(void);
-extern void flip_ranges(struct range_t *__restrict ranges, struct range_t *__restrict tmp_ranges)
+extern void flip_ranges(struct range_t *restrict flip_me, struct range_t *restrict tmp_ranges)
     __attribute__ ((nonnull(1, 2)));
 /* support functions */
 extern int (*parse_ipaddr) (const char *restrict src, union ipaddr_t *restrict dst);
 extern int parse_ipaddr_init(const char *restrict src,
-			     union ipaddr_t *restrict dst) _DP_ATTRIBUTE_CONST;
+			     union ipaddr_t *restrict dst);
 extern int parse_ipaddr_v4(const char *restrict src, union ipaddr_t *restrict dst);
 extern int parse_ipaddr_v6(const char *restrict src, union ipaddr_t *restrict dst);
 
+extern void parse_cidr(struct range_t *range_p, const char *word);
+
 extern void (*copy_ipaddr) (union ipaddr_t *restrict dst, const union ipaddr_t *restrict src);
 extern void copy_ipaddr_init(union ipaddr_t *restrict dst,
-			     const union ipaddr_t *restrict src) _DP_ATTRIBUTE_CONST;
+			     const union ipaddr_t *restrict src);
 extern void copy_ipaddr_v4(union ipaddr_t *restrict dst, const union ipaddr_t *restrict src);
 extern void copy_ipaddr_v6(union ipaddr_t *restrict dst, const union ipaddr_t *restrict src);
 
 extern const char *(*ntop_ipaddr) (const union ipaddr_t *ip);
-extern const char *ntop_ipaddr_init(const union ipaddr_t *ip) _DP_ATTRIBUTE_CONST;
+extern const char *ntop_ipaddr_init(const union ipaddr_t *ip);
 extern const char *ntop_ipaddr_v4(const union ipaddr_t *ip);
 extern const char *ntop_ipaddr_v6(const union ipaddr_t *ip);
 
 extern double (*get_range_size) (const struct range_t *r);
-extern double get_range_size_init(const struct range_t *r) _DP_ATTRIBUTE_CONST;
-extern double get_range_size_v4(const struct range_t *r) _DP_ATTRIBUTE_PURE;
-extern double get_range_size_v6(const struct range_t *r) _DP_ATTRIBUTE_PURE;
+extern double get_range_size_init(const struct range_t *r);
+extern double get_range_size_v4(const struct range_t *r);
+extern double get_range_size_v6(const struct range_t *r);
 
-extern int (*xstrstr) (const char *__restrict str);
-extern int xstrstr_init(const char *__restrict str) _DP_ATTRIBUTE_CONST;
-extern int xstrstr_v4(const char *__restrict str)
-_DP_ATTRIBUTE_HOT _DP_ATTRIBUTE_PURE;
-extern int xstrstr_v6(const char *__restrict str)
-_DP_ATTRIBUTE_HOT _DP_ATTRIBUTE_PURE;
+extern int (*xstrstr) (const char *restrict str);
+extern int xstrstr_init(const char *restrict str);
+extern int xstrstr_v4(const char *restrict str)
+_DP_ATTRIBUTE_HOT;
+extern int xstrstr_v6(const char *restrict str)
+_DP_ATTRIBUTE_HOT;
 
-extern double strtod_or_err(const char *__restrict str, const char *__restrict errmesg);
-extern void print_version(void) __attribute__ ((noreturn));
-extern void usage(int status) __attribute__ ((noreturn));
+extern double strtod_or_err(const char *restrict str, const char *restrict errmesg);
+extern void __attribute__ ((noreturn)) print_version(void);
+extern void __attribute__ ((noreturn)) usage(int status);
 /* qsort required functions... */
 /* ...for ranges and... */
 extern int (*ipcomp) (const union ipaddr_t *restrict a, const union ipaddr_t *restrict b);
 extern int ipcomp_init(const union ipaddr_t *restrict a,
-		       const union ipaddr_t *restrict b) _DP_ATTRIBUTE_CONST;
+		       const union ipaddr_t *restrict b);
 extern int ipcomp_v4(const union ipaddr_t *restrict a,
-		     const union ipaddr_t *restrict b) _DP_ATTRIBUTE_PURE;
+		     const union ipaddr_t *restrict b);
 extern int ipcomp_v6(const union ipaddr_t *restrict a,
-		     const union ipaddr_t *restrict b) _DP_ATTRIBUTE_PURE;
+		     const union ipaddr_t *restrict b);
 
 extern int (*leasecomp) (const struct leases_t *restrict a, const struct leases_t *restrict b);
 extern int leasecomp_init(const struct leases_t *restrict a
@@ -291,23 +283,23 @@ extern int leasecomp_init(const struct leases_t *restrict a
 extern int leasecomp_v4(const struct leases_t *restrict a, const struct leases_t *restrict b);
 extern int leasecomp_v6(const struct leases_t *restrict a, const struct leases_t *restrict b);
 
-extern int comp_cur(struct range_t *r1, struct range_t *r2) _DP_ATTRIBUTE_PURE;
-extern int comp_double(double f1, double f2) _DP_ATTRIBUTE_CONST;
+extern int comp_cur(struct range_t *r1, struct range_t *r2);
+extern int comp_double(double f1, double f2);
 extern int comp_ip(struct range_t *r1, struct range_t *r2);
 extern int comp_max(struct range_t *r1, struct range_t *r2);
 extern int comp_percent(struct range_t *r1, struct range_t *r2);
-extern int comp_tc(struct range_t *r1, struct range_t *r2) _DP_ATTRIBUTE_PURE;
+extern int comp_tc(struct range_t *r1, struct range_t *r2);
 extern int comp_tcperc(struct range_t *r1, struct range_t *r2);
-extern int comp_touched(struct range_t *r1, struct range_t *r2) _DP_ATTRIBUTE_PURE;
-extern int rangecomp(const void *__restrict r1, const void *__restrict r2)
+extern int comp_touched(struct range_t *r1, struct range_t *r2);
+extern int rangecomp(const void *restrict r1, const void *restrict r2)
     __attribute__ ((nonnull(1, 2)));
 /* sort function pointer and functions */
 extern comparer_t field_selector(char c);
 extern double ret_percent(struct range_t r);
-extern double ret_tc(struct range_t r) _DP_ATTRIBUTE_CONST;
+extern double ret_tc(struct range_t r);
 extern double ret_tcperc(struct range_t r);
-extern void mergesort_ranges(struct range_t *__restrict orig, int size,
-			     struct range_t *__restrict temp)
+extern void mergesort_ranges(struct range_t *restrict orig, int size,
+			     struct range_t *restrict temp)
     __attribute__ ((nonnull(1, 3)));
 /* output function pointer and functions */
 extern int (*output_analysis) (void);
@@ -320,15 +312,15 @@ extern int output_alarming(void);
 /* Memory release, file closing etc */
 extern void clean_up(void);
 /* Hash functions */
-extern void (*add_lease) (union ipaddr_t *ip, enum ltype type);
-extern void add_lease_init(union ipaddr_t *ip, enum ltype type) _DP_ATTRIBUTE_CONST;
-extern void add_lease_v4(union ipaddr_t *ip, enum ltype type);
-extern void add_lease_v6(union ipaddr_t *ip, enum ltype type);
+extern void (*add_lease) (union ipaddr_t *addr, enum ltype type);
+extern void add_lease_init(union ipaddr_t *addr, enum ltype type);
+extern void add_lease_v4(union ipaddr_t *addr, enum ltype type);
+extern void add_lease_v6(union ipaddr_t *addr, enum ltype type);
 
-extern struct leases_t *(*find_lease) (union ipaddr_t *ip);
-extern struct leases_t *find_lease_init(union ipaddr_t *ip) _DP_ATTRIBUTE_CONST;
-extern struct leases_t *find_lease_v4(union ipaddr_t *ip) _DP_ATTRIBUTE_PURE;
-extern struct leases_t *find_lease_v6(union ipaddr_t *ip) _DP_ATTRIBUTE_PURE;
+extern struct leases_t *(*find_lease) (union ipaddr_t *addr);
+extern struct leases_t *find_lease_init(union ipaddr_t *addr);
+extern struct leases_t *find_lease_v4(union ipaddr_t *addr);
+extern struct leases_t *find_lease_v6(union ipaddr_t *addr);
 
 extern void delete_lease(struct leases_t *lease);
 extern void delete_all_leases(void);
