@@ -44,7 +44,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <netinet/in.h>
-#include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -65,17 +64,13 @@ int parse_leases(void)
 	char *line, *ipstring, macstring[20], *stop;
 	union ipaddr_t addr;
 	struct stat lease_file_stats;
-	bool ethernets = false;
+	int ethernets = 0;	/* boolean */
 	struct leases_t *lease;
 
 	dhcpd_leases = fopen(config.dhcpdlease_file, "r");
 	if (dhcpd_leases == NULL)
 		error(EXIT_FAILURE, errno, "parse_leases: %s", config.dhcpdlease_file);
 #ifdef HAVE_POSIX_FADVISE
-# ifdef POSIX_FADV_NOREUSE
-	if (posix_fadvise(fileno(dhcpd_leases), 0, 0, POSIX_FADV_NOREUSE) != 0)
-		error(EXIT_FAILURE, errno, "parse_leases: fadvise %s", config.dhcpdlease_file);
-# endif				/* POSIX_FADV_NOREUSE */
 # ifdef POSIX_FADV_SEQUENTIAL
 	if (posix_fadvise(fileno(dhcpd_leases), 0, 0, POSIX_FADV_SEQUENTIAL) != 0)
 		error(EXIT_FAILURE, errno, "parse_leases: fadvise %s", config.dhcpdlease_file);
@@ -92,7 +87,7 @@ int parse_leases(void)
 	ipstring = xmalloc(sizeof(char) * MAXLEN);
 	ipstring[0] = '\0';
 	if (config.print_mac_addreses == 1)
-		ethernets = true;
+		ethernets = 1;
 	while (!feof(dhcpd_leases)) {
 		if (!fgets(line, MAXLEN, dhcpd_leases) && ferror(dhcpd_leases))
 			error(EXIT_FAILURE, errno, "parse_leases: %s", config.dhcpdlease_file);
@@ -128,10 +123,10 @@ int parse_leases(void)
 			if ((lease = find_lease(&addr)) != NULL)
 				delete_lease(lease);
 			add_lease(&addr, BACKUP);
-			config.backups_found = true;
+			config.backups_found = 1;
 			break;
 		case PREFIX_HARDWARE_ETHERNET:
-			if (ethernets == false)
+			if (ethernets == 0)
 				break;
 			memcpy(macstring, line + 20, 17);
 			macstring[17] = '\0';
@@ -158,11 +153,30 @@ static int is_interesting_config_clause(char const *restrict s)
 		return ITS_A_RANGE_FIRST_IP;
 	if (strstr(s, "shared-network"))
 		return ITS_A_SHAREDNET;
+	if (config.all_as_shared) {
+		if (strstr(s, "subnet"))
+			return ITS_A_SUBNET;
+		if (strstr(s, "netmask"))
+			return ITS_A_NETMASK;
+	}
 	if (strstr(s, "include"))
 		return ITS_AN_INCLUCE;
 	return ITS_NOTHING_INTERESTING;
 }
 
+/*! \brief Flip first and last IP in range if they are in unusual order.
+ */
+static void reorder_last_first(struct range_t *range_p)
+{
+	if (ipcomp(&range_p->first_ip, &range_p->last_ip) > 0) {
+		union ipaddr_t tmp;
+
+		tmp = range_p->first_ip;
+		range_p->first_ip = range_p->last_ip;
+		range_p->last_ip = tmp;
+	}
+}
+
 /*! \brief The dhcpd.conf file parser.
  * FIXME: This spaghetti monster function need to be rewrote at least
  * ones.
@@ -171,7 +185,7 @@ void parse_config(int is_include, const char *restrict config_file,
 		  struct shared_network_t *restrict shared_p)
 {
 	FILE *dhcpd_config;
-	bool newclause = true, comment = false, one_ip_range = false;
+	int newclause = 1, comment = 0, one_ip_range = 0; /* booleans */
 	int quote = 0, braces = 0, argument = ITS_NOTHING_INTERESTING;
 	size_t i = 0;
 	char *word;
@@ -188,10 +202,6 @@ void parse_config(int is_include, const char *restrict config_file,
 	if (dhcpd_config == NULL)
 		error(EXIT_FAILURE, errno, "parse_config: %s", config_file);
 #ifdef HAVE_POSIX_FADVISE
-# ifdef POSIX_FADV_NOREUSE
-	if (posix_fadvise(fileno(dhcpd_config), 0, 0, POSIX_FADV_NOREUSE) != 0)
-		error(EXIT_FAILURE, errno, "parse_config: fadvise %s", config_file);
-# endif				/* POSIX_FADV_NOREUSE */
 # ifdef POSIX_FADV_SEQUENTIAL
 	if (posix_fadvise(fileno(dhcpd_config), 0, 0, POSIX_FADV_SEQUENTIAL) != 0)
 		error(EXIT_FAILURE, errno, "parse_config: fadvise %s", config_file);
@@ -207,10 +217,10 @@ void parse_config(int is_include, const char *restrict config_file,
 			/* Handle comments if they are not quoted */
 		case '#':
 			if (quote == 0)
-				comment = true;
+				comment = 1;
 			continue;
 		case '"':
-			if (comment == false) {
+			if (comment == 0) {
 				quote++;
 				/* Either one or zero */
 				quote = quote % 2;
@@ -220,18 +230,18 @@ void parse_config(int is_include, const char *restrict config_file,
 			/* New line resets comment section, but
 			 * not if quoted */
 			if (quote == 0)
-				comment = false;
+				comment = 0;
 			break;
 		case ';':
 			/* Quoted colon does not mean new clause */
 			if (0 < quote)
 				break;
-			if (comment == false
+			if (comment == 0
 			    && argument != ITS_A_RANGE_FIRST_IP
 			    && argument != ITS_A_RANGE_SECOND_IP && argument != ITS_AN_INCLUCE) {
-				newclause = true;
+				newclause = 1;
 				i = 0;
-			} else if (argument == ITS_A_RANGE_FIRST_IP && one_ip_range == true) {
+			} else if (argument == ITS_A_RANGE_FIRST_IP && one_ip_range == 1) {
 				argument = ITS_A_RANGE_SECOND_IP;
 				c = ' ';
 			} else if (argument == ITS_A_RANGE_SECOND_IP && 0 < i) {
@@ -245,8 +255,9 @@ void parse_config(int is_include, const char *restrict config_file,
 				break;
 			} else if (argument == ITS_A_RANGE_SECOND_IP && i == 0) {
 				if (!range_p) {
-					puts("parse_config: range_p uninitialized: report a bug");
-					abort();
+					long int pos;
+					pos = ftell(dhcpd_config);
+					error(EXIT_FAILURE, 0, "parse_config: parsing failed at position: %ld", pos);
 				}
 				range_p->last_ip = range_p->first_ip;
 				goto newrange;
@@ -261,14 +272,14 @@ void parse_config(int is_include, const char *restrict config_file,
 			 *
 			 * shared-network DSL{ ... */
 			if (i == 0) {
-				newclause = true;
+				newclause = 1;
 				continue;
 			} else
 				break;
 		case '}':
 			if (0 < quote)
 				break;
-			if (comment == false) {
+			if (comment == 0) {
 				braces--;
 				/* End of shared-network */
 				if (braces_shared == braces) {
@@ -277,23 +288,23 @@ void parse_config(int is_include, const char *restrict config_file,
 					braces_shared = 1000;
 					shared_p = shared_networks;
 				}
-				/* Not literally true, but works for this
+				/* Not literally 1, but works for this
 				 * program */
-				newclause = true;
+				newclause = 1;
 			}
 			continue;
 		default:
 			break;
 		}
 		/* Either inside comment or Nth word of clause. */
-		if (comment == true || (newclause == false && argument == ITS_NOTHING_INTERESTING))
+		if (comment == 1 || (newclause == 0 && argument == ITS_NOTHING_INTERESTING))
 			continue;
 		/* Strip white spaces before new clause word. */
-		if ((newclause == true || argument != ITS_NOTHING_INTERESTING)
-		    && isspace(c) && i == 0 && one_ip_range == false)
+		if ((newclause == 1 || argument != ITS_NOTHING_INTERESTING)
+		    && isspace(c) && i == 0 && one_ip_range == 0)
 			continue;
 		/* Save to word which clause this is. */
-		if ((newclause == true || argument != ITS_NOTHING_INTERESTING)
+		if ((newclause == 1 || argument != ITS_NOTHING_INTERESTING)
 		    && (!isspace(c) || 0 < quote)) {
 			word[i] = c;
 			i++;
@@ -301,26 +312,26 @@ void parse_config(int is_include, const char *restrict config_file,
 			 * of words are this long which the program is
 			 * searching. */
 			if (MAXLEN == i) {
-				newclause = false;
+				newclause = 0;
 				i = 0;
 				continue;
 			}
 		}
 		/* See if clause is something that parser is looking for. */
-		else if (newclause == true) {
+		else if (newclause == 1) {
 			/* Insert string end & set state */
 			word[i] = '\0';
 			if (word[i - 1] != '{')
-				newclause = false;
+				newclause = 0;
 			i = 0;
 			argument = is_interesting_config_clause(word);
 			if (argument == ITS_A_RANGE_FIRST_IP)
-				one_ip_range = true;
+				one_ip_range = 1;
 		}
 		/* words after range, shared-network or include */
 		else if (argument != ITS_NOTHING_INTERESTING) {
 			word[i] = '\0';
-			newclause = false;
+			newclause = 0;
 			i = 0;
 
 			switch (argument) {
@@ -328,12 +339,19 @@ void parse_config(int is_include, const char *restrict config_file,
 				/* printf ("range 2nd ip: %s\n", word); */
 				range_p = ranges + num_ranges;
 				argument = ITS_NOTHING_INTERESTING;
-				parse_ipaddr(word, &addr);
-				if (one_ip_range == true) {
-					one_ip_range = false;
-					copy_ipaddr(&range_p->first_ip, &addr);
+				if (strchr(word, '/')) {
+					parse_cidr(range_p, word);
+					one_ip_range = 0;
+				} else {
+					/* not cidr */
+					parse_ipaddr(word, &addr);
+					if (one_ip_range == 1) {
+						one_ip_range = 0;
+						copy_ipaddr(&range_p->first_ip, &addr);
+					}
+					copy_ipaddr(&range_p->last_ip, &addr);
+					reorder_last_first(range_p);
 				}
-				copy_ipaddr(&range_p->last_ip, &addr);
  newrange:
 				range_p->count = 0;
 				range_p->touched = 0;
@@ -345,7 +363,7 @@ void parse_config(int is_include, const char *restrict config_file,
 					ranges = xrealloc(ranges, sizeof(struct range_t) * RANGES);
 					range_p = ranges + num_ranges;
 				}
-				newclause = true;
+				newclause = 1;
 				break;
 			case ITS_A_RANGE_FIRST_IP:
 				/* printf ("range 1nd ip: %s\n", word); */
@@ -354,10 +372,16 @@ void parse_config(int is_include, const char *restrict config_file,
 					/* word was not ip, try again */
 					break;
 				copy_ipaddr(&range_p->first_ip, &addr);
-				one_ip_range = false;
+				one_ip_range = 0;
 				argument = ITS_A_RANGE_SECOND_IP;
 				break;
 			case ITS_A_SHAREDNET:
+			case ITS_A_SUBNET:
+				/* ignore subnets inside a shared-network */
+				if (argument == ITS_A_SUBNET && shared_p != shared_networks) {
+					argument = ITS_NOTHING_INTERESTING;
+					break;
+				}
 				/* printf ("shared-network named: %s\n", word); */
 				num_shared_networks++;
 				shared_p = shared_networks + num_shared_networks;
@@ -366,18 +390,40 @@ void parse_config(int is_include, const char *restrict config_file,
 				shared_p->used = 0;
 				shared_p->touched = 0;
 				shared_p->backups = 0;
+				shared_p->netmask = (argument == ITS_A_SUBNET ? -1 : 0); /* do not fill in netmask */
 				if (SHARED_NETWORKS < num_shared_networks + 2)
 					/* FIXME: make this to go away by reallocating more space. */
 					error(EXIT_FAILURE, 0,
 					      "parse_config: increase default.h SHARED_NETWORKS and recompile");
+				/* record network's mask too */
+				if (argument == ITS_A_SUBNET)
+				        newclause = 1;
+				argument = ITS_NOTHING_INTERESTING;
+				braces_shared = braces;
+				break;
+			case ITS_A_NETMASK:
+				/* fill in only when requested to do so */
+				if (shared_p->netmask) {
+					if (!(parse_ipaddr(word, &addr)))
+						break;
+					shared_p->netmask = 32;
+					while ((addr.v4 & 0x01) == 0) {
+						addr.v4 >>= 1;
+						shared_p->netmask--;
+					}
+					snprintf(word, MAXLEN-1, "%s/%d", shared_p->name, shared_p->netmask);
+					if (shared_p->name)
+						free(shared_p->name);
+					shared_p->name = xstrdup(word);
+				}
 				argument = ITS_NOTHING_INTERESTING;
 				braces_shared = braces;
 				break;
 			case ITS_AN_INCLUCE:
 				/* printf ("include file: %s\n", word); */
 				argument = ITS_NOTHING_INTERESTING;
-				parse_config(false, word, shared_p);
-				newclause = true;
+				parse_config(0, word, shared_p);
+				newclause = 1;
 				break;
 			case ITS_NOTHING_INTERESTING:
 				/* printf ("nothing interesting: %s\n", word); */
