@@ -43,6 +43,7 @@
 #include <errno.h>
 #include <inttypes.h>
 #include <langinfo.h>
+#include <locale.h>
 #include <math.h>
 #include <netinet/in.h>
 #include <stdio.h>
@@ -433,7 +434,6 @@ int output_json(void)
 		fprintf(outfile, "         \"free\":%g\n",
 			shared_networks->available - shared_networks->used);
 		fprintf(outfile, "   }");	/* end of summary */
-		sep++;
 	}
 
 	fprintf(outfile, "\n}\n");
@@ -469,6 +469,8 @@ static void html_header(FILE *restrict f)
 	if (tmp == NULL) {
 		error(EXIT_FAILURE, errno, "html_header: localtime");
 	}
+	setlocale(LC_CTYPE, "");
+	setlocale(LC_NUMERIC, "");
 	if (strftime(outstr, sizeof(outstr), nl_langinfo(D_T_FMT), &result) == 0) {
 		error(EXIT_FAILURE, 0, "html_header: strftime returned 0");
 	}
@@ -1002,8 +1004,12 @@ int output_alarming(void)
 	} else {
 		if (config.number_limit & A_BIT)
 			fprintf(outfile, "OK:");
-		else
+		else {
+			if (close_stream(outfile)) {
+				error(EXIT_FAILURE, errno, "output_alarming: fclose");
+			}
 			return ret_val;
+		}
 	}
 	if (config.header_limit & R_BIT) {
 		fprintf(outfile, " Ranges - crit: %d warn: %d ok: %d", rc, rw, ro);
@@ -1014,6 +1020,29 @@ int output_alarming(void)
 		if (ri != 0) {
 			fprintf(outfile, " range_ignored=%d", ri);
 		}
+		if (config.perfdata == 1 && config.number_limit & R_BIT) {
+			for (i = 0; i < num_ranges; i++) {
+				range_p--;
+				range_size = get_range_size(range_p);
+				if (config.minsize < range_size) {
+					fprintf(outfile, " %s_r=",
+						ntop_ipaddr(&range_p->first_ip));
+					fprintf(outfile, "%g;%g;%g;0;%g",
+						range_p->count,
+						(range_size * config.warning / 100),
+						(range_size * config.critical / 100),
+						range_size);
+					fprintf(outfile, " %s_rt=%g",
+						ntop_ipaddr(&range_p->first_ip),
+						range_p->touched);
+					if (config.backups_found == 1) {
+						fprintf(outfile, " %s_rbu=%g",
+							ntop_ipaddr(&range_p->first_ip),
+							range_p->backups);
+					}
+				}
+			}
+		}
 		fprintf(outfile, "\n");
 	} else {
 		fprintf(outfile, " ");
@@ -1025,7 +1054,29 @@ int output_alarming(void)
 		}
 		fprintf(outfile, "; | snet_crit=%d snet_warn=%d snet_ok=%d", sc, sw, so);
 		if (si != 0) {
-			fprintf(outfile, " snet_ignored=%d\n", si);
+			fprintf(outfile, " snet_ignored=%d", si);
+		}
+		if (config.perfdata == 1 && config.header_limit & R_BIT) {
+			for (i = 0; i < num_shared_networks; i++) {
+				if (config.minsize < shared_p->available) {
+					fprintf(outfile, " '%s_s'=%g;%g;%g;0;%g",
+						shared_p->name,
+						shared_p->used,
+						(shared_p->available * config.warning / 100),
+						(shared_p->available * config.critical / 100),
+						shared_p->available);
+					fprintf(outfile, " '%s_st'=%g",
+						shared_p->name,
+						shared_p->touched);
+					if (config.backups_found == 1) {
+						fprintf(outfile, " '%s_sbu'=%g",
+						shared_p->name,
+						shared_p->backups);
+					}
+				}
+				shared_p--;
+			}
+			fprintf(outfile, "\n");
 		}
 	}
 	fprintf(outfile, "\n");
